searchNodes=[{"doc":"ECSx is an Entity-Component-System (ECS) framework for Elixir. In ECS: Every game object is an Entity, represented by a unique ID. The data which comprises an Entity is split among many Components. Game logic is split into Systems, which update the Components every server tick. Components are grouped together - with similar Components from other Entities - into categories called Aspects. Systems can then be organized in a way where each one only operates on Components of a specific Aspect. Under the hood, ECSx uses Erlang Term Storage (ETS) to store active Components in memory. A single GenServer manages the ETS tables to ensure strict serializability and customize the run order for Systems.","ref":"ECSx.html","title":"ECSx","type":"module"},{"doc":"Aspects provide an API for working with Components of a specific type. For example, if Entities in your application should have a &quot;color&quot; value, you will create an Aspect called Color . This allows you to add a color Component to an Entity with add_component/1 , query the color value for a given Entity with get_component/1 or get_value/2 , query all Entities which have a color value with get_all/0 , remove the color value from an Entity altogether with remove_component/1 , or test whether an entity has a color with has_component?/1 . Usage Each Aspect should have its own module, which defines a schema for the data, and optional ETS table settings. The schema defines the field names which will be used to reference each value. defmodule MyApp.Aspects.Color do use ECSx.Aspect , schema : { :entity_id , :hue , :saturation , :lightness } end Options :schema - a tuple of field names :table_type - ETS table type. Possible options are :set , :ordered_set , :bag , and :duplicate_bag . Defaults to :set . See below for more information. :read_concurrency - when true , enables read concurrency for the ETS table. Defaults to false :table_type option By default, ECSx creates ETS tables of type :set for your Components. This means that each Entity may have at most one Component of each Aspect. For example, if your application has a &quot;Height&quot; Aspect, Entities would never need more than one Component to model that value. However, if you had an Aspect such as &quot;TakingDamageOverTime&quot;, you might want an Entity to store each source of damage as a separate value, using multiple Components of the same Aspect. To do this, set :table_type to :bag . Example defmodule TakingDamageOverTime do use ECSx.Aspect , schema : { :entity_id , :damage_per_second , :damage_type , :source_id } , table_type : :bag end alias TakingDamageOverTime , as : DOT DOT . add_component ( entity_id : hero . id , damage_per_second : 10 , type : :poison , source_id : spider . id ) DOT . add_component ( entity_id : hero . id , damage_per_second : 25 , type : :fire , source_id : dragon . id ) DOT . get_component ( hero . id ) [ %{ entity_id : ... } , %{ ... } ] DOT . get_value ( hero . id , :damage_per_second ) [ 10 , 25 ] get_component/1 and get_value/2 The standard way to fetch the component(s) from an Entity is using MyAspect.get_component(entity_id) . If your table_type is :set or :ordered_set , this function will return a single Component, or nil if the Entity does not have that Aspect. If the table_type is :bag or :duplicate_bag , then it will return a list containing zero or more Components. Each Component is represented by a map, using the keys provided by the schema. If you only want a single value from the Component, use get_value/2 , passing the field name of the desired value as the second argument. Examples Color . get_component ( entity_id ) %{ entity_id : entity_id , hue : 300 , saturation : 50 , lightness : 45 } Color . get_value ( entity_id , :hue ) 300","ref":"ECSx.Aspect.html","title":"ECSx.Aspect","type":"behaviour"},{"doc":"Creates a new component. Example ArmorRating . add_component ( entity_id : 123 , value : 10 )","ref":"ECSx.Aspect.html#c:add_component/1","title":"ECSx.Aspect.add_component/1","type":"callback"},{"doc":"Gets all existing components of this aspect.","ref":"ECSx.Aspect.html#c:get_all/0","title":"ECSx.Aspect.get_all/0","type":"callback"},{"doc":"Gets an existing component from the table, given its entity ID.","ref":"ECSx.Aspect.html#c:get_component/1","title":"ECSx.Aspect.get_component/1","type":"callback"},{"doc":"Gets a single value from a component.","ref":"ECSx.Aspect.html#c:get_value/2","title":"ECSx.Aspect.get_value/2","type":"callback"},{"doc":"Checks if an entity has one or more components with this aspect.","ref":"ECSx.Aspect.html#c:has_component?/1","title":"ECSx.Aspect.has_component?/1","type":"callback"},{"doc":"Initializes the ETS table to store all components of this aspect.","ref":"ECSx.Aspect.html#c:init/0","title":"ECSx.Aspect.init/0","type":"callback"},{"doc":"","ref":"ECSx.Aspect.html#init/3","title":"ECSx.Aspect.init/3","type":"function"},{"doc":"Removes any existing components of this aspect from an entity.","ref":"ECSx.Aspect.html#c:remove_component/1","title":"ECSx.Aspect.remove_component/1","type":"callback"},{"doc":"","ref":"ECSx.Aspect.html#t:aspect/0","title":"ECSx.Aspect.aspect/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:id/0","title":"ECSx.Aspect.id/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:t/0","title":"ECSx.Aspect.t/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:value/0","title":"ECSx.Aspect.value/0","type":"type"},{"doc":"A Component labels an entity as possessing a particular aspect, and holds the data needed to model that aspect. Under the hood, we use ETS to store the Components in memory for quick retrieval via aspect and entity ID.","ref":"ECSx.Component.html","title":"ECSx.Component","type":"module"},{"doc":"","ref":"ECSx.Component.html#add/3","title":"ECSx.Component.add/3","type":"function"},{"doc":"","ref":"ECSx.Component.html#exists?/2","title":"ECSx.Component.exists?/2","type":"function"},{"doc":"","ref":"ECSx.Component.html#get_all/2","title":"ECSx.Component.get_all/2","type":"function"},{"doc":"","ref":"ECSx.Component.html#get_many/3","title":"ECSx.Component.get_many/3","type":"function"},{"doc":"","ref":"ECSx.Component.html#get_one/3","title":"ECSx.Component.get_one/3","type":"function"},{"doc":"","ref":"ECSx.Component.html#get_value/4","title":"ECSx.Component.get_value/4","type":"function"},{"doc":"","ref":"ECSx.Component.html#get_values/4","title":"ECSx.Component.get_values/4","type":"function"},{"doc":"","ref":"ECSx.Component.html#remove/2","title":"ECSx.Component.remove/2","type":"function"},{"doc":"In an ECSx application, the Manager is responsible for: starting up ETS tables for each Aspect, where the Components will be stored keeping track of the Systems to run, and their run order configuring the tick rate for the application running the Systems every tick validating and writing updates from the player clients","ref":"ECSx.Manager.html","title":"ECSx.Manager","type":"module"},{"doc":"","ref":"ECSx.Manager.html#final_tick/1","title":"ECSx.Manager.final_tick/1","type":"function"},{"doc":"","ref":"ECSx.Manager.html#setup/1","title":"ECSx.Manager.setup/1","type":"macro"},{"doc":"","ref":"ECSx.Manager.html#start_link/1","title":"ECSx.Manager.start_link/1","type":"function"},{"doc":"A system is a fragment of game logic which acts on all entities with the desired components. Every system must implement a run function. By default the system will run every game tick. To use a longer period between runs, you can pass the option :period . For example, to set a system to run every 5 ticks: use ECSx.System , period : 5","ref":"ECSx.System.html","title":"ECSx.System","type":"behaviour"},{"doc":"","ref":"ECSx.System.html#c:run/0","title":"ECSx.System.run/0","type":"callback"},{"doc":"Helpers for the ECSx mmix tasks.","ref":"Mix.Tasks.ECSx.Helpers.html","title":"Mix.Tasks.ECSx.Helpers","type":"task"},{"doc":"","ref":"Mix.Tasks.ECSx.Helpers.html#otp_app/0","title":"Mix.Tasks.ECSx.Helpers.otp_app/0","type":"function"},{"doc":"","ref":"Mix.Tasks.ECSx.Helpers.html#root_module/0","title":"Mix.Tasks.ECSx.Helpers.root_module/0","type":"function"},{"doc":"","ref":"Mix.Tasks.ECSx.Helpers.html#write_file/2","title":"Mix.Tasks.ECSx.Helpers.write_file/2","type":"function"},{"doc":"Generates a new Aspect for an ECSx application. $ mix ecsx.gen.aspect","ref":"Mix.Tasks.Ecsx.Gen.Aspect.html","title":"Mix.Tasks.Ecsx.Gen.Aspect","type":"task"},{"doc":"Generates a new System for an ECSx application. $ mix ecsx.gen.system Foo","ref":"Mix.Tasks.Ecsx.Gen.System.html","title":"Mix.Tasks.Ecsx.Gen.System","type":"task"},{"doc":"Generates the manager process which runs an ECSx application. $ mix ecsx.setup","ref":"Mix.Tasks.Ecsx.Setup.html","title":"Mix.Tasks.Ecsx.Setup","type":"task"},{"doc":"To create an ECSx application, there are a few simple steps: install Elixir + erlang/OTP create an Elixir project fetch ECSx as a dependency for your project run the ECSx setup","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"If you don't yet have Elixir and erlang/OTP installed on your machine, follow the instructions on the official Installation Page .","ref":"installation.html#elixir-and-erlang-otp","title":"Installation - Elixir and erlang/OTP","type":"extras"},{"doc":"To create a new Elixir application with supervision tree, run the following command: $ mix new my_app --sup Where my_app will be the name of your app. If you want to use the Phoenix Web Framework , instead run (after hex and phx_new are installed): $ mix phx.new my_app","ref":"installation.html#create-project","title":"Installation - Create project","type":"extras"},{"doc":"To use the ECSx framework in your application, it should be added to the list of dependencies in my_app/mix.exs : defp deps do [ { :ecsx , &quot;~&gt; 0.1&quot; } ] end Then (from the root directory of your application) run: $ mix deps.get","ref":"installation.html#install-ecsx","title":"Installation - Install ECSx","type":"extras"},{"doc":"With ECSx installed, you can run the setup generator: $ mix ecsx.setup which will create the Manager, a sample Aspect, and a sample System to get your project started.","ref":"installation.html#setup-ecsx","title":"Installation - Setup ECSx","type":"extras"},{"doc":"You should now have everything you need to start building! If you're already familiar with the Entity-Component-System pattern, jump right in to the tutorial project - otherwise, start with our guide on ECS design .","ref":"installation.html#summary","title":"Installation - Summary","type":"extras"},{"doc":"Coming soon!","ref":"tutorial.html","title":"Tutorial Project","type":"extras"},{"doc":"Coming soon!","ref":"ecs_design.html","title":"ECS Design","type":"extras"}]